@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,6 +38,8 @@ exception statement from your version. */
 
 package java.awt;
 
+import java.awt.image.BufferStrategy;
+
 /**
  * A double-buffering capability descriptor.  This class exposes
  * details about the double-buffering algorithms used by image
@@ -60,7 +62,7 @@ package java.awt;
  * @author Eric Blake (ebb9@email.byu.edu)
  *
  * @see BufferStrategy#getCapabilities()
- * @see GraphicsConfiguration#getCapabilities()
+ * @see GraphicsConfiguration#getBufferCapabilities()
  *
  * @since 1.4
  */
