@@ -1,5 +1,5 @@
 /* ByteBuffer.java -- 
-   Copyright (C) 2002, 2003, 2004 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -15,8 +15,8 @@ General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with GNU Classpath; see the file COPYING.  If not, write to the
-Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
-02111-1307 USA.
+Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+02110-1301 USA.
 
 Linking this library statically or dynamically with other modules is
 making a combined work based on this library.  Thus, the terms and
@@ -38,20 +38,24 @@ exception statement from your version. */
 
 package java.nio;
 
+import gnu.classpath.Pointer;
+
 /**
  * @since 1.4
  */
 public abstract class ByteBuffer extends Buffer
-  implements Comparable
+  implements Comparable<ByteBuffer>
 {
   ByteOrder endian = ByteOrder.BIG_ENDIAN;
+  final byte[] backing_buffer;
+  final int array_offset;
 
-  int array_offset;
-  byte[] backing_buffer;
-
-  ByteBuffer (int capacity, int limit, int position, int mark)
+  ByteBuffer (int capacity, int limit, int position, int mark,
+	      Pointer address, byte[] backing_buffer, int array_offset)
   {
-    super (capacity, limit, position, mark);
+    super (capacity, limit, position, mark, address);
+    this.backing_buffer = backing_buffer;
+    this.array_offset = array_offset;
   }
 
   /**
@@ -77,7 +81,7 @@ public abstract class ByteBuffer extends Buffer
    * @exception IndexOutOfBoundsException If the preconditions on the offset
    * and length parameters do not hold
    */
-  final public static ByteBuffer wrap (byte[] array, int offset, int length)
+  public static final ByteBuffer wrap (byte[] array, int offset, int length)
   {
     // FIXME: In GCJ and other implementations where arrays may not
     // move we might consider, at least when offset==0:
@@ -94,7 +98,7 @@ public abstract class ByteBuffer extends Buffer
    * Wraps a <code>byte</code> array into a <code>ByteBuffer</code>
    * object.
    */
-  final public static ByteBuffer wrap (byte[] array)
+  public static final ByteBuffer wrap (byte[] array)
   {
     return wrap (array, 0, array.length);
   }
@@ -260,11 +264,27 @@ public abstract class ByteBuffer extends Buffer
 
   /**
    * Calculates a hash code for this buffer.
+   *
+   * This is done with <code>int</code> arithmetic,
+   * where ** represents exponentiation, by this formula:<br>
+   * <code>s[position()] + 31 + (s[position()+1] + 30)*31**1 + ... +
+   * (s[limit()-1]+30)*31**(limit()-1)</code>.
+   * Where s is the buffer data. Note that the hashcode is dependent
+   * on buffer content, and therefore is not useful if the buffer
+   * content may change.
+   *
+   * @return the hash code
    */
   public int hashCode ()
   {
-    // FIXME: Check what SUN calculates here.
-    return super.hashCode ();
+    int hashCode = get(position()) + 31;
+    int multiplier = 1;
+    for (int i = position() + 1; i < limit(); ++i)
+      {
+	  multiplier *= 31;
+	  hashCode += (get(i) + 30)*multiplier;
+      }
+    return hashCode;
   }
 
   /**
@@ -274,7 +294,7 @@ public abstract class ByteBuffer extends Buffer
   {
     if (obj instanceof ByteBuffer)
       {
-        return compareTo (obj) == 0;
+        return compareTo ((ByteBuffer) obj) == 0;
       }
 
     return false;
@@ -286,10 +306,8 @@ public abstract class ByteBuffer extends Buffer
    * @exception ClassCastException If obj is not an object derived from
    * <code>ByteBuffer</code>.
    */
-  public int compareTo (Object obj)
+  public int compareTo (ByteBuffer other)
   {
-    ByteBuffer other = (ByteBuffer) obj;
-
     int num = Math.min(remaining(), other.remaining());
     int pos_this = position();
     int pos_other = other.position();
