@@ -1,5 +1,5 @@
 /* ShortBuffer.java -- 
-   Copyright (C) 2002, 2003, 2004  Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,19 +38,24 @@ exception statement from your version. */
 
 package java.nio;
 
+import gnu.classpath.Pointer;
+
 /**
  * @since 1.4
  */
 public abstract class ShortBuffer extends Buffer
-  implements Comparable
+  implements Comparable<ShortBuffer>
 {
-  int array_offset;
-  short[] backing_buffer;
+  final int array_offset;
+  final short[] backing_buffer;
 
-  ShortBuffer (int capacity, int limit, int position, int mark)
+  ShortBuffer (int capacity, int limit, int position,
+	       int mark, Pointer address, short[] backing_buffer,
+	       int array_offset)
   {
-    super (capacity, limit, position, mark);
-    array_offset = 0;
+    super (capacity, limit, position, mark, address);
+    this.backing_buffer = backing_buffer;
+    this.array_offset = array_offset;
   }
 
   /**
@@ -273,7 +278,7 @@ public abstract class ShortBuffer extends Buffer
   {
     if (obj instanceof ShortBuffer)
       {
-        return compareTo (obj) == 0;
+        return compareTo ((ShortBuffer) obj) == 0;
       }
 
     return false;
@@ -285,10 +290,8 @@ public abstract class ShortBuffer extends Buffer
    * @exception ClassCastException If obj is not an object derived from
    * <code>ShortBuffer</code>.
    */
-  public int compareTo (Object obj)
+  public int compareTo (ShortBuffer other)
   {
-    ShortBuffer other = (ShortBuffer) obj;
-
     int num = Math.min(remaining(), other.remaining());
     int pos_this = position();
     int pos_other = other.position();
