@@ -40,7 +40,6 @@ exception statement from your version. */
 #include "gtkpeer.h"
 #include "gnu_java_awt_peer_gtk_GtkToolkit.h"
 #include "jcl.h"
-#include <gdk/gdkx.h>
 
 #define RC_FILE ".classpath-gtkrc"
 
@@ -79,9 +78,6 @@ exception statement from your version. */
 #define VK_CAPS_LOCK 20
 #define VK_META 157
 
-struct state_table *cp_gtk_native_state_table;
-struct state_table *cp_gtk_native_global_ref_table;
-
 static jclass gtkgenericpeer;
 static jclass gtktoolkit;
 static JavaVM *java_vm;
@@ -129,6 +125,13 @@ static void glog_func (const gchar *log_domain,
 		       gpointer user_data);
 #endif
 
+JNIEXPORT void JNICALL
+Java_gnu_java_awt_peer_gtk_GtkToolkit_initIDs
+(JNIEnv *env, jclass cls __attribute__((unused)))
+{
+  gtkpeer_init_pointer_IDs(env);
+}
+
 /*
  * Call gtk_init.  It is very important that this happen before any other
  * gtk calls.
@@ -158,8 +161,6 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
   printCurrentThreadID = (*env)->GetStaticMethodID (env, gtkgenericpeer,
                                                     "printCurrentThread", "()V");
  
-  NSA_INIT (env, gtkgenericpeer);
-
   g_assert((*env)->GetJavaVM(env, &java_vm) == 0);
 
   /* GTK requires a program's argc and argv variables, and requires that they
@@ -182,10 +183,6 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
 
   gtk_widget_set_default_colormap (gdk_rgb_get_colormap ());
 
-  /* Make sure queued calls don't get sent to GTK/GDK while 
-     we're shutting down. */
-  atexit (gdk_threads_enter);
-
   if ((homedir = getenv ("HOME")))
     {
       rcpath = (char *) g_malloc (strlen (homedir) + strlen (RC_FILE) + 2);
@@ -203,7 +200,7 @@ Java_gnu_java_awt_peer_gtk_GtkToolkit_gtkInit (JNIEnv *env,
   old_glog_func = g_log_set_default_handler (&glog_func, NULL);
 #endif
 
-  cp_gtk_button_init_jni ();
+  cp_gtk_button_init_jni (env);
   cp_gtk_checkbox_init_jni ();
   cp_gtk_choice_init_jni ();
   cp_gtk_component_init_jni ();
