@@ -1,5 +1,5 @@
 /* FloatSeqHolder.java --
-   Copyright (C) 2005 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2006 Free Software Foundation, Inc.
 
 This file is part of GNU Classpath.
 
@@ -38,7 +38,7 @@ exception statement from your version. */
 
 package org.omg.CORBA;
 
-import gnu.CORBA.primitiveArrayTypeCode;
+import gnu.CORBA.typecodes.ArrayTypeCode;
 
 import org.omg.CORBA.portable.InputStream;
 import org.omg.CORBA.portable.OutputStream;
@@ -62,8 +62,8 @@ public final class FloatSeqHolder
   /**
    * The type code for this holder. Each holder has a different instance.
    */
-  private final primitiveArrayTypeCode typecode =
-    new primitiveArrayTypeCode(TCKind.tk_float);
+  private final ArrayTypeCode typecode =
+    new ArrayTypeCode(TCKind.tk_float);
 
   /**
    * Constructs an instance of FloatSeqHolder,
@@ -87,7 +87,7 @@ public final class FloatSeqHolder
   }
 
   /**
-   * Fill in the {@link value } field by reading the required data
+   * Fill in the {@link #value } field by reading the required data
    * from the given stream. This method first reads the array size
    * (as CORBA <code>long</code>) and then calls the
    * {@link org.omg.CORBA.portable.InputStream#read_float_array }.
@@ -111,7 +111,7 @@ public final class FloatSeqHolder
   }
 
   /**
-   * Write the {@link value } field to the given stream.
+   * Write the {@link #value } field to the given stream.
    * This method first writes the array size
    * (as CORBA <code>long</code>) and then calls the
    * {@link org.omg.CORBA.portable.OutputStream#write_float_array }.
